sha512sum /home/circleci/project/girder/setup.py \
    /home/circleci/project/girder/requirements-dev.txt \
    /home/circleci/project/girder/pytest_girder/setup.py \
    /home/circleci/project/girder/plugins/*/setup.py \
    /home/circleci/project/girder/clients/python/setup.py
