.g-task-progress-bar
  width 140px
  display inline-block
  margin-bottom 0
  margin-right 7px
  border-radius 2px

  .progress-bar
    transition none
    position relative
  .progress-status
    position absolute
    width 140px
    color #222
    text-shadow 0 0 2px white
    text-align center
    line-height 20px
    font-size 12px
    font-weight bold

    .progress-percent
      display inline
    .progress-left
      display inline
      margin-left 10px

.g-task-progress-title
  display inline-block
  color #222
  line-height 20px
  vertical-align top
  margin 0
  padding 0
  width 220px
  overflow hidden
  text-overflow ellipsis
  white-space nowrap

.g-task-progress-message
  color #888
  font-size 12px
